;---------------------------------------------------
; EX_HIO ; Hybrid input output algorism
; u : image (complex), ua : |u|, up : u/|u| (phase)
; F : FT(u), Fa : |F|,  Fp : F/|F| (phase)
; Intro. IDL program. (2013): Chapt. 6.8  
; Author:  Tadashi Ito     
;---------------------------------------------------
pro b1_ev, event
;--- recover uv0
widget_control, event.top, get_uvalue=uv0, /no_copy
;---imaginal unit
im=complex(0,1);
;--- get slider-1(iter)
widget_control, uv0.s1_id, get_value=iter
;---get slider-2(beta) 
widget_control, uv0.s2_id, get_value=beta
;---check image size
sz=size(*uv0.data)
xsize=sz[1]
ysize=sz[2]
;---support size
mul=4
spsizx=xsize*mul
spsizy=ysize*mul
;---build support 
u_0 = make_array(spsizx, spsizy, /complex,value = 0.) 
;--- place image somewhere in support 
xpos=200
ypos=50
u_0[xpos,ypos]= *uv0.data
;--- Initial condition
F_0 = fft(u_0)          ;complex amplitude
F0a = abs(F_0)          ;amplitude
F0p = atan(F_0,/phase)  ;Phase
;--- initialise variables for reconstruction 
;--- array to hold error 
err=make_array(iter+1, /float,value = 0.)
;--- image in real space during iteration
u = make_array(spsizx,spsizy,/complex, value = 0.)
;--- image in fourier space during iteration 
F = make_array(spsizx,spsizy,/complex,value = 0.)
;--- initial phase 
Fp = make_array(spsizx,spsizy,/float,value = 1.)
;--- mask array for constraint condition
mask = make_array(spsizx,spsizy,/byte,value = 1);
;--- construct initial fourier amplitude 
F = F0a*exp(im*Fp)
;--- back to real space for iteration
u = fft(F, /inverse) 
;--- image area 
ximag=xpos+xsize
yimag=ypos+ysize
;--- now iterating for the reconstruction
for j = 0L, iter do begin
u_sav = u
;--- display phase of initial image
F=fft(u);
wset, uv0.w2
tvscl,congrid(atan(F,/phase),!d.x_size,!d.y_size)
;--- replace fourier amplitude to initial one
F=F0a*(F/abs(F))
;-- back to real space
u = fft(F, /inverse)
;--- implementation of constraints
test = (imaginary(u) gt 0.)
mask[xpos:ximag,ypos:yimag]=test[xpos:ximag,ypos:yimag]
rmask = abs(1-mask)
;--- plot error
wset, uv0.w4
erj= sqrt(total(total(abs(u*mask)^2,1)))
if j eq 0 then er0=erj
err[j]=erj/er0
plot, err,/ylog,yrange=[0.0001,1.]
;---calculate new real image by HIO
u = mask*(u_sav-(beta*u)) + rmask*u
;---display new image
ua = abs(u)
wset, uv0.w3
tvscl,congrid(ua[xpos:ximag,ypos:yimag],!d.x_size,!d.y_size)
;---check stop button
sb = widget_info(uv0.b2_id,/button_set)
if sb ne 0 then break
endfor
;---clear button & set uvalue of base widget
    widget_control,uv0.b2_id,set_button=0
    widget_control,event.top,set_uvalue=uv0,/no_copy
end
;-----------------
pro b2_ev, event 
; Stop called
end
;-----------------
pro b3_ev, event 
; Exit called
widget_control, Event.top, /DESTROY 
end 
;-------------------------
pro EX_HIO_event, event 
; treat other events
end
;---------------------------------------
PRO EX_HIO
;---------------------------------------
g0_id=widget_base(title='EX_HIO',/row)
g1_id=widget_base(g0_id, /frame, /column)
g2_id=widget_base(g0_id, /frame, /column)
g3_id=widget_base(g0_id, /frame, /column)
;---display wondow size
wsizx=256
wsizy=256
;---first group (image)
d1_id=widget_draw(g1_id,uname='d1',xsize=wsizx,ysize=wsizy)
d2_id=widget_draw(g1_id,uname='d2',xsize=wsizx,ysize=wsizy)
;---second group (phase)
d3_id=widget_draw(g2_id,uname='d3',xsize=wsizx,ysize=wsizy)
d4_id=widget_draw(g2_id,uname='d4',xsize=wsizx,ysize=wsizy)
;---fourth group (button & sliders)
s1_id=widget_slider(g3_id,value=1000,minimum=1,maximum=1000, $
title='iteration',uname='S1')
s2_id=cw_fslider(g3_id,value=0.8,minimum=0.,maximum=1., $
title='beta',uname='S2',/drag)
b1_id=widget_button(g3_id,value='Start',uname='b1',event_pro='b1_ev',ysize=30)
b2_id=widget_button(g3_id,value='Stop',uname='b2',event_pro='b2_ev',ysize=30)
b3_id=widget_button(g3_id,value='Exit',uname='b3',event_pro='b3_ev',ysize=30)
;---realize wigets
widget_control,g0_id, /realize
;---get window indices 
widget_control,d1_id,get_value=w1
widget_control,d2_id,get_value=w2
widget_control,d3_id,get_value=w3
widget_control,d4_id,get_value=w4
;---state structure
uv0={data:ptr_new(),b1_id:b1_id,b2_id:b2_id,b3_id:b3_id, $
s1_id:s1_id,s2_id:s2_id,w1:w1,w2:w2,w3:w3,w4:w4}
;---read grey image
file=filepath('mineral.png',subdir='examples\data')
if (file eq '') then return
read_png,file,data
;---save in heap area
uv0.data=ptr_new(data,/no_copy)
;---draw data
wset,uv0.w1  
tvscl, congrid(*uv0.data,!d.x_size,!d.y_size) 
;---set uvalue of base widget
widget_control,g0_id,set_uvalue=uv0,/no_copy
;---call event manager
xmanager,'EX_HIO',g0_id,cleanup='myclean',/no_block
end