;-------------------------------------------------
; OBJexWig ;  Object Widgets
; Intro. IDL program. (2013): Chapt. 7.6 
; Author:  Tadashi Ito     
;-------------------------------------------------
pro objexwig::b1_ev, ev
self.rotflg=0b
end
;-------------------------
pro objexwig::b2_ev, ev
self.rotflg=1b
end
;-------------------------
pro objexwig::b3_ev, ev
self.omodel->setproperty, transform=self.tm0
self.owin->draw, self.oview
end
;-------------------------
pro objexwig::b4_ev, ev
widget_control, /destroy, ev.top
end
;-------------------------
pro objexwig::d1_ev, ev
;--- type=4:expose
if (ev.type eq 4) then begin
    self.owin->draw, self.oview
    return
    endif
;--- handle trackball updates
if self.rotflg eq 0b then trans = self.otrack->update(ev,transform=qm) $
    else trans = self.otrack->update(ev,transform=qm,/translate)
if (trans ne 0) then begin
    self.omodel->getproperty, transform=tm
    self.omodel->setproperty, transform=tm#qm
    self.owin->draw, self.oview
    endif
;--- handle other events
;--- 0:button down
if (ev.type eq 0) then begin
    self.btndown = 1b
    widget_control, self.d1_id, draw_motion=1
    self.owin->draw, self.oview
    endif
;--- 1:button up
if (ev.type eq 1) then begin
    if (self.btndown eq 1b) then $
    self.owin->draw, self.oview
    self.btndown = 0b
    widget_control, self.d1_id, draw_motion=0
    endif
end          
;---------------------------
pro objexwig::cleanup,base
obj_destroy,[self.owin,self.oview,self.omodel,self.oatom,self.otrack]
end
;---------------------------
pro callclean,base
widget_control, base, get_uvalue = ouv
ouv->cleanup,base
help,/heap
end
;---------------------------
pro callevt,ev
widget_control, ev.top, get_uvalue = ouv
uname=widget_info(ev.id,/uname)
case uname of
    'b1': ouv->b1_ev,ev
    'b2': ouv->b2_ev,ev
    'b3': ouv->b3_ev,ev
    'b4': ouv->b4_ev,ev
    'd1': ouv->d1_ev,ev
    else:
    endcase
end 
;---------------------------
pro objexwig::callXm
xmanager,'objexwig',self.g0_id,event_handler='callevt',cleanup='callclean',/no_block
end
;---------------------------
pro objexwig::drawGr
;--- create a view (white)
self.oview = mkoview(color=[255,255,255])
;--- create a model
self.omodel = mkomodel()
self.oview->add, self.omodel
;--- create a surface atom object
self.omodel->add, self.oatom
;--- scale and center
get_bounds, self.oatom, xr, yr, zr
xc=[0.,1./(xr[1]-xr[0])]
yc=[0.,1./(yr[1]-yr[0])]
zc=[0.,1./(zr[1]-zr[0])]
self.oatom->setproperty,xcoord_conv=xc,ycoord_conv=yc,zcoord_conv=zc
;--- initial direction
self.omodel->rotate, [1,0,0],-80
self.omodel->rotate, [0,1,0], 20
self.omodel->rotate, [1,0,0], 30
self.owin->draw,self.oview
self.omodel->getproperty, transform=tm
self.tm0=tm
end
;----------------------------
pro objexwig::realzWig
;--- realize widget 
widget_control, self.g0_id, /realize
widget_control, self.d1_id, get_value=ow
self.owin=ow
end
;----------------------------
pro objexwig::drawWig
self.g0_id=widget_base(title='ObjectWidgets',/column,uvalue=self)
g1_id=widget_base(self.g0_id,/row)
bmp_r=filepath('rotate.bmp',subdir=['resource','bitmaps'])
bmp_p=filepath('pan.bmp',subdir=['resource','bitmaps'])
bmp_0=filepath('view.bmp',subdir=['resource','bitmaps'])
bmp_s=filepath('stop.bmp',subdir=['resource','bitmaps'])
b1_id=widget_button(g1_id,value=bmp_r,/bitmap, uname='b1')
b2_id=widget_button(g1_id,value=bmp_p,/bitmap, uname='b2')
b3_id=widget_button(g1_id,value=bmp_0,/bitmap, uname='b3')
b4_id=widget_button(g1_id,value=bmp_s,/bitmap, uname='b4')
device,get_screen_size=ssize
ws=ssize[1]*0.5
self.d1_id=widget_draw(self.g0_id,uname='d1',xsize=ws,ysize=ws,retain=0,  $
   /expose_events,/button_events,graphics_level=2) 
;---  trackball object
self.otrack = obj_new('trackball',[ws*0.5, ws*0.5], ws*0.5)
end   
;-------------------------------
function objexwig::init,oatom
self.oatom=oatom
self->drawWig
self->realzWig
self->drawGr
self->callXm
return, 1
end
;-------------------------------
function objexwig,oatom
if n_params() eq 0 then oatom=mkosinc(style=1)
return, obj_new('objexwig',oatom)
end
;-------------------------------
pro objexwig__define
struct={objexwig, owin:obj_new(), oview:obj_new(),  $
omodel:obj_new(),oatom:obj_new(),otrack:obj_new(), $
g0_id:0L, d1_id:0L, rotflg:0b, btndown:0b, tm0:dblarr(4,4)}
end
